/* Einfaches Beispiel fuer die Anwendung von Swing und AWT.
Demonstration der Model-View-Controller-Architektur.
Vorlesung "Softwaretechnologie".
(c) Heinrich Hussmann, TU Dresden 1997-2000, LMU Mnchen 2005 

Variante mit anonymen Klassen als Button-Listener */

import java.util.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

class WindowCloser extends WindowAdapter implements WindowListener {

	 public void windowClosing(WindowEvent event) {
	 	System.exit(0);
	 }
}

// The View

class CounterFrame extends JFrame implements Observer {

    private Counter ctr;

	JPanel valuePanel = new JPanel();
	JTextField valueDisplay = new JTextField(10);
	
	JButton countButton = new JButton("Count");
	JButton resetButton = new JButton("Reset");
	JButton exitButton = new JButton("Exit");

	JPanel buttonPanel = new JPanel();

	public CounterFrame (Counter c) {
	 	setTitle("Counter");
	 	ctr = c;
	 	
	 	valuePanel.add(new Label("Counter value"));
	 	valuePanel.add(valueDisplay);
	 	valueDisplay.setEditable(false);
	 	valueDisplay.setText(String.valueOf(ctr.getValue()));
	 	getContentPane().add(valuePanel,BorderLayout.NORTH);
	 	
	 	countButton.addActionListener(new ActionListener() {
	 	    public void actionPerformed (ActionEvent event) {
	 	        ctr.count();
	 	    }
	 	});
		buttonPanel.add(countButton);
	 	resetButton.addActionListener(new ActionListener() {
	 	    public void actionPerformed (ActionEvent event) {
	 	        ctr.reset();
	 	    }
	 	});
	 	buttonPanel.add(resetButton);
	 	exitButton.addActionListener(new ActionListener() {
	 	    public void actionPerformed (ActionEvent event) {
	 	        System.exit(0);
	 	    }
	 	});
	 	buttonPanel.add(exitButton);
	 	getContentPane().add(buttonPanel,BorderLayout.SOUTH);
	 	
	 	addWindowListener(new WindowAdapter()
	 	  {public void windowClosing(WindowEvent event) {
	 	    System.exit(0);
	      }
	    });
	 	c.addObserver(this);
	 	
	 	pack();
	 	setVisible(true);
	 }
	 
	 public void update (Observable o, Object arg) {
	 	valueDisplay.setText(String.valueOf(ctr.getValue()));
	 }
}


// The Model

class Counter extends Observable {

	private int k = 0;
	
	public void count () {
		k++;
		setChanged();
		notifyObservers();
	}
	
	public void reset () {
		k = 0;
		setChanged();
		notifyObservers();
	}
	
	public int getValue () {
		return k;
	}
}

class MVCx3 {

public static void main (String[] argv) {	
  // Look and Feel kann durch Angabe eines (beliebigen) Arguments auf der Kommandozeile umgeschaltet werden
  // Also z.B. "java MVCx test liefert anderes Look And Feel"
    try {
      if (argv.length == 0)
   	UIManager.setLookAndFeel(
    	  UIManager.getCrossPlatformLookAndFeelClassName());
      else
        UIManager.setLookAndFeel(
          UIManager.getSystemLookAndFeelClassName());
    } catch (Exception e) {}
	CounterFrame cf = new CounterFrame(new Counter());
	}
}
